<?php

namespace FiloBlu\Storelocator\Model\Google;

class Api {

    const CONFIG_FILE_DIR_PREFIX = 'filoblu/storelocator/';

    private $client = null;
    protected $helper = null;
    protected $session = null;
    protected $request = null;
    protected $config = null;
    protected $storeManager = null;
    protected $directory_list = null;
    protected $_workingTimes = null;

    public function __construct(
    \Magento\Backend\Helper\Data $helper, \Magento\Framework\Session\SessionManagerInterface $session, \Magento\Framework\App\RequestInterface $request, \Magento\Framework\App\Config\ScopeConfigInterface $config, \Magento\Store\Model\StoreManagerInterface $storeManager, \Magento\Framework\App\Filesystem\DirectoryList $directory_list, \FiloBlu\Storelocator\Model\WorkingTimes $workingTimes
    ) {

        $this->helper = $helper;
        $this->session = $session;
        $this->request = $request;
        $this->config = $config;
        $this->storeManager = $storeManager;
        $this->directory_list = $directory_list;
        $this->_workingTimes = $workingTimes;

        require_once(dirname(dirname(dirname(__DIR__))) . '/google-api-php-client/google/autoload.php');
        require_once(dirname(dirname(dirname(__DIR__))) . '/google-api-php-client/google/MyBusiness.php');
    }

    /**
     * Get Client
     *
     * @return Google_Client
     */
    public function getClient() {

        if (!$this->client) {
            $app_name = $this->config->getValue('filoblu_storelocator_section/google_my_business/gmb_api_app_name');

            $this->client = new \Google_Client();
            $this->client->setApplicationName($app_name);

            $this->client->setClientId($this->config->getValue('filoblu_storelocator_section/google_my_business/gmb_api_client_id'));
            $this->client->setClientSecret($this->config->getValue('filoblu_storelocator_section/google_my_business/gmb_api_client_secret'));
            $this->client->setAccessType('offline');
            $this->client->setApprovalPrompt('force');

            $token = $this->getAuthData();

            if ($token) {

                $this->client->setAccessToken($token);

                $token = json_decode($token, true);
                if ($this->client->isAccessTokenExpired()) {
                    //refresh access token
                    $this->client->refreshToken($token['refresh_token']);
                    $updated_token = json_decode($this->client->getAccessToken(), true);
                    //keep the original refresh_token for later reuse
                    $updated_token['refresh_token'] = $token['refresh_token'];
                    $this->storeAuthData(json_encode($updated_token));
                }
            }

            $this->client->setState
                    (
                    $this->helper->getUrl("adminhtml/system_config/edit", array('section' => 'filoblu_storelocator_section'))
            );

            $scopes = array("https://www.googleapis.com/auth/plus.business.manage");

            $this->client->setScopes($scopes);

            $this->client->setRedirectUri($this->config->getValue('filoblu_storelocator_section/google_my_business/gmb_redirect_uri'));

            if (isset($_GET['code'])) {
                $this->getClient()->authenticate($_GET['code']);

                $updated_token = json_decode($this->client->getAccessToken(), true);

                //keep the original refresh_token for later reuse
                $updated_token['refresh_token'] = $updated_token['refresh_token'];

                $this->storeAuthData(json_encode($updated_token));

                header('Location: ' . $this->helper->getUrl("adminhtml/system_config/edit", array('section' => 'filoblu_storelocator_section')));
                exit();
            }
        }

        return $this->client;
    }

    private function storeAuthData($token) {
        if (file_put_contents($this->directory_list->getPath('media') . DIRECTORY_SEPARATOR . SELF::CONFIG_FILE_DIR_PREFIX . "oauth_token.json", json_encode($token)) > 0) {
            return true;
        } else {
            throw new Exception("Could not write the OAuth token to disk");
        }
    }

    private function getAuthData() {

        $filename = $this->directory_list->getPath('media') . DIRECTORY_SEPARATOR . SELF::CONFIG_FILE_DIR_PREFIX . "oauth_token.json";

        if (file_exists($filename)) {
            return json_decode(file_get_contents($filename), true);
        } else {
            return false;
        }
    }

    public function getCategoryList($regionCode = "IT", $languageCode = "it-IT") {

        try {
            $api = new \Google_Service_MyBusiness($this->getClient());
            $result = $api->categories->listCategories(array("regionCode" => $regionCode, "languageCode" => $languageCode));

            $array = array();
            foreach ($result->getCategories() as $categoryObj) {
                $array[$categoryObj->getCategoryId()] = $categoryObj->getName();
            }

            asort($array);

            return $array;
        } catch (Exception $ex) {
            return [];
        } catch (\Google_Service_Exception $ex) {
            return [];
        } catch (\Google_Auth_Exception $ex) {
            return [];
        }
    }

    public function getLocationObj($store) {
        $location = new \Google_Service_MyBusiness_Location();
//        $location->setName($store->getTitle());
        $languageCode = "en";

        if ($store->getGmbLanguageCode()) {
            $languageCode = $store->getGmbLanguageCode();
        }

        $location->setLanguageCode($languageCode);
        $location->setStoreCode($store->getId());
        $location->setLocationName($store->getTitle());
        $location->setPrimaryPhone($store->getPhone1());
        $location->setAdditionalPhones($store->getPhone2());
        // Address
        $address = new \Google_Service_MyBusiness_Address();
        $address->setAddressLines($store->getStreet());
        $address->setLocality($store->getCity());
        $address->setAdministrativeArea($store->getCity());
        $address->setCountry($store->getCountry());
        $address->setPostalCode($store->getPostcode());
        $location->setAddress($address);

        // Primary category
        $primaryCategory = new \Google_Service_Mybusiness_Category();
        $primaryCategory->setCategoryId($store->getGmbCat());
        $location->setPrimaryCategory($primaryCategory);

        $location->setWebsiteUrl($store->getWebsiteUrl());

        // Lat LNg
        $latlng = new \Google_Service_MyBusiness_LatLng();
        $latlng->setLatitude($store->getLatitude());
        $latlng->setLongitude($store->getLongitude());
        $location->setLatlng($latlng);

        // Hours
        $hours = new \Google_Service_MyBusiness_BusinessHours();
        $hoursArray = [];

        $workingTimesCollection = $this->_workingTimes->loadByStoreId($store->getId());
        $workingTimesCollectionArray = $workingTimesCollection->getData();
        $wtIndex = 0;
        $skipNextForContinuedTimetable = false;

        foreach ($workingTimesCollectionArray as $workingTime) {

            if ($skipNextForContinuedTimetable) {
                $skipNextForContinuedTimetable = false;
                $wtIndex++;
                continue;
            }
            $isClosed = $workingTime["closed"];

            if ($isClosed || (!$workingTime["from"] && !$workingTime["to"])) {
                $wtIndex++;
                continue;
            }

            $fromHH = "00";
            $fromMM = "00";
            $fromSS = "00";

            if ($workingTime["from"]) {
                $timeFrom = explode(":", explode(" ", $workingTime["from"])[1]);
                $fromHH = $timeFrom[0];
                $fromMM = $timeFrom[1];
                $fromSS = $timeFrom[2];

                // Closed day
                if ($fromHH == "00" && $fromMM == "00" && isset($workingTimesCollectionArray[$wtIndex + 1])) {
                    $wtNext = $workingTimesCollectionArray[$wtIndex + 1];
                    if ($wtNext["day"] == $workingTime["day"]) {
                        if ($wtNext["to"]) {
                            $timeToNext = explode(":", explode(" ", $wtNext["to"])[1]);
                            $toHHNext = $timeToNext[0];
                            $toMMNext = $timeToNext[1];
                            $toSSNext = $timeToNext[2];
                            if ($toHHNext == "00" && $toMMNext == "00") {
                                $wtIndex++;
                                continue;
                            }
                        }
                    }
                }
            }

            $toHH = "00";
            $toMM = "00";
            $toSS = "00";

            if ($workingTime["to"]) {
                $timeTo = explode(":", explode(" ", $workingTime["to"])[1]);
                $toHH = $timeTo[0];
                $toMM = $timeTo[1];
                $toSS = $timeTo[2];

                // Closed day
                if ($toHH == "00" && $toMM == "00" && isset($workingTimesCollectionArray[$wtIndex - 1])) {
                    $wtPrev = $workingTimesCollectionArray[$wtIndex - 1];
                    if ($wtPrev["day"] == $workingTime["day"]) {
                        if ($wtPrev["from"]) {
                            $timeFromPrev = explode(":", explode(" ", $wtPrev["from"])[1]);
                            $fromHHPrev = $timeFromPrev[0];
                            $fromMMPrev = $timeFromPrev[1];
                            $fromSSPrev = $timeFromPrev[2];
                            if ($fromHHPrev == "00" && $fromMMPrev == "00") {
                                $wtIndex++;
                                continue;
                            }
                        }
                    }
                }

                // Continued timetable
                if ($toHH == "00" && $toMM == "00" && isset($workingTimesCollectionArray[$wtIndex + 1])) {
                    $wtNext = $workingTimesCollectionArray[$wtIndex + 1];
                    if ($wtNext["day"] == $workingTime["day"]) {

                        if ($wtNext["to"]) {
                            $timeToNext = explode(":", explode(" ", $wtNext["to"])[1]);
                            $toHHNext = $timeToNext[0];
                            $toMMNext = $timeToNext[1];
                            $toSSNext = $timeToNext[2];
                        } else {
                            $toHHNext = "00";
                            $toMMNext = "00";
                            $toSSNext = "00";
                        }

                        if (!$wtNext["from"]) {
                            $toHH = $toHHNext;
                            $toMM = $toMMNext;
                            $toSS = $toSSNext;

                            $skipNextForContinuedTimetable = true;
                        }

                        if ($wtNext["from"]) {
                            $timeFromNext = explode(":", explode(" ", $wtNext["from"])[1]);
                            $fromHHNext = $timeFromNext[0];
                            $fromMMNext = $timeFromNext[1];
                            $fromSSNext = $timeFromNext[2];
                            if ($fromHHNext == "00" && $fromMMNext == "00") {
                                $skipNextForContinuedTimetable = true;
                                $toHH = $toHHNext;
                                $toMM = $toMMNext;
                                $toSS = $toSSNext;
                            }
                        }
                    }
                }
            }

            $dayStr = '';

            switch ($workingTime["day"]) {
                case 1:
                    $dayStr = "MONDAY";
                    break;
                case 2:
                    $dayStr = "TUESDAY";
                    break;
                case 3:
                    $dayStr = "WEDNESDAY";
                    break;
                case 4:
                    $dayStr = "THURSDAY";
                    break;
                case 5:
                    $dayStr = "FRIDAY";
                    break;
                case 6:
                    $dayStr = "SATURDAY";
                    break;
                case 7:
                    $dayStr = "SUNDAY";
                    break;

                default:
                    $dayStr = "DAY_OF_WEEK_UNSPECIFIED";
                    break;
            }

            $hoursArray[] = [
                "openDay" => $dayStr,
                "closeDay" => $dayStr,
                "openTime" => $fromHH . ':' . $fromMM,
                "closeTime" => $toHH . ':' . $toMM
            ];

            $wtIndex++;
        }

        if (count($hoursArray)) {

            $hours->setPeriods($hoursArray);
            $location->setRegularHours($hours);
        }

        return $location;
    }

    public function createLocation(\FiloBlu\Storelocator\Model\Stores $store) {

        try {
            $api = new \Google_Service_MyBusiness($this->getClient());
            $location = $this->getLocationObj($store);
            $result = $api->accounts_locations->create($this->config->getValue('filoblu_storelocator_section/google_my_business/gmb_account_name'), $location, array("requestId" => uniqid()));

            return $result;
        } catch (Exception $ex) {
            return $ex->getMessage();
        } catch (\Google_Service_Exception $ex) {
            return $ex->getMessage();
        } catch (\Google_Auth_Exception $ex) {
            $ex->getMessage();
        }
    }

    public function updateLocation($name, \FiloBlu\Storelocator\Model\Stores $store) {

        try {
            $api = new \Google_Service_MyBusiness($this->getClient());
            $location = $this->getLocationObj($store);
            $result = $api->accounts_locations->patch($name, $location);

            return $result;
        } catch (Exception $ex) {
            return $ex->getMessage();
        } catch (\Google_Service_Exception $ex) {
            return $ex->getMessage();
        } catch (\Google_Auth_Exception $ex) {
            $ex->getMessage();
        }
    }

    public function deleteLocation($name) {

        try {
            $api = new \Google_Service_MyBusiness($this->getClient());
            $result = $api->accounts_locations->delete($name);

            return $result;
        } catch (Exception $ex) {
            return $ex->getMessage();
        } catch (\Google_Service_Exception $ex) {
            return $ex->getMessage();
        } catch (\Google_Auth_Exception $ex) {
            $ex->getMessage();
        }
    }

    public function getLocation($name) {

        try {
            $api = new \Google_Service_MyBusiness($this->getClient());
            $result = $api->accounts_locations->get($name);

            return $result;
        } catch (Exception $ex) {
            return $ex->getMessage();
        } catch (\Google_Service_Exception $ex) {
            return $ex->getMessage();
        } catch (\Google_Auth_Exception $ex) {
            $ex->getMessage();
        }
    }

    public function getAccountList() {

        try {
            $api = new \Google_Service_MyBusiness($this->getClient());
            $result = $api->accounts->listAccounts();

            return $result;
        } catch (Exception $ex) {
            return $ex->getMessage();
        } catch (\Google_Service_Exception $ex) {
            return $ex->getMessage();
        } catch (\Google_Auth_Exception $ex) {
            $ex->getMessage();
        }
    }

}
